package org.apache.bcel.classfile;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import org.apache.commons.lang3.ArrayUtils;

public final class ExceptionTable extends Attribute {
   private int[] exceptionIndexTable;

   public ExceptionTable(ExceptionTable c) {
      this(c.getNameIndex(), c.getLength(), c.getExceptionIndexTable(), c.getConstantPool());
   }

   ExceptionTable(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
      this(nameIndex, length, (int[])null, constantPool);
      int exceptionCount = input.readUnsignedShort();
      this.exceptionIndexTable = new int[exceptionCount];

      for(int i = 0; i < exceptionCount; ++i) {
         this.exceptionIndexTable[i] = input.readUnsignedShort();
      }

   }

   public ExceptionTable(int nameIndex, int length, int[] exceptionIndexTable, ConstantPool constantPool) {
      super((byte)3, nameIndex, length, constantPool);
      this.exceptionIndexTable = exceptionIndexTable != null ? exceptionIndexTable : ArrayUtils.EMPTY_INT_ARRAY;
   }

   public void accept(Visitor v) {
      v.visitExceptionTable(this);
   }

   public Attribute copy(ConstantPool constantPool) {
      ExceptionTable c = (ExceptionTable)this.clone();
      if (this.exceptionIndexTable != null) {
         c.exceptionIndexTable = (int[])this.exceptionIndexTable.clone();
      }

      c.setConstantPool(constantPool);
      return c;
   }

   public void dump(DataOutputStream file) throws IOException {
      super.dump(file);
      file.writeShort(this.exceptionIndexTable.length);
      int[] var2 = this.exceptionIndexTable;
      int var3 = var2.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         int index = var2[var4];
         file.writeShort(index);
      }

   }

   public int[] getExceptionIndexTable() {
      return this.exceptionIndexTable;
   }

   public String[] getExceptionNames() {
      String[] names = new String[this.exceptionIndexTable.length];
      Arrays.setAll(names, (i) -> {
         return Utility.pathToPackage(super.getConstantPool().getConstantString(this.exceptionIndexTable[i], (byte)7));
      });
      return names;
   }

   public int getNumberOfExceptions() {
      return this.exceptionIndexTable == null ? 0 : this.exceptionIndexTable.length;
   }

   public void setExceptionIndexTable(int[] exceptionIndexTable) {
      this.exceptionIndexTable = exceptionIndexTable != null ? exceptionIndexTable : ArrayUtils.EMPTY_INT_ARRAY;
   }

   public String toString() {
      StringBuilder buf = new StringBuilder();
      buf.append("Exceptions: ");

      for(int i = 0; i < this.exceptionIndexTable.length; ++i) {
         String str = super.getConstantPool().getConstantString(this.exceptionIndexTable[i], (byte)7);
         buf.append(Utility.compactClassName(str, false));
         if (i < this.exceptionIndexTable.length - 1) {
            buf.append(", ");
         }
      }

      return buf.toString();
   }
}
